#import "NSString+WMFPageUtilities.h"
#import "NSString+WMFExtras.h"

NSString *const WMFInternalLinkPathPrefix = @"/wiki/";

NSString *const WMFReferenceFragmentSubstring = @"ref_";
NSString *const WMFCitationFragmentSubstring = @"cite_note";
NSString *const WMFEndNoteFragmentSubstring = @"endnote_";

@implementation NSString (WMFPageUtilities)

- (BOOL)wmf_isWikiResource {
    return [self containsString:WMFInternalLinkPathPrefix];
}

- (BOOL)wmf_isReferenceFragment {
    return [self containsString:WMFReferenceFragmentSubstring];
}

- (BOOL)wmf_isCitationFragment {
    return [self containsString:WMFCitationFragmentSubstring];
}

- (BOOL)wmf_isEndNoteFragment {
    return [self containsString:WMFEndNoteFragmentSubstring];
}

- (NSString *)wmf_pathWithoutWikiPrefix {
    NSRange internalLinkRange = [self rangeOfString:WMFInternalLinkPathPrefix];
    NSString *path = internalLinkRange.location == NSNotFound ? self : [self wmf_safeSubstringFromIndex:WMFRangeGetMaxIndex(internalLinkRange)];
    return [path precomposedStringWithCanonicalMapping];
}

- (NSString *)wmf_unescapedNormalizedPageTitle {
    return [[self stringByRemovingPercentEncoding] wmf_normalizedPageTitle];
}

- (NSString *)wmf_normalizedPageTitle {
    return [[self stringByReplacingOccurrencesOfString:@"_" withString:@" "] precomposedStringWithCanonicalMapping];
}

- (NSString *)wmf_denormalizedPageTitle {
    return [[self stringByReplacingOccurrencesOfString:@" " withString:@"_"] precomposedStringWithCanonicalMapping];
}

@end
